# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .billing_model import BillingModel

__all__ = ["BillingModelListResponse"]

BillingModelListResponse: TypeAlias = List[BillingModel]
